# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gtk, time, gobject
import Attachment as AttachmentCore, GNOME, GConf, Conf__, Conf, i18n, func, \
	Window, Handler, Menu, WEE, NotesItem

_ = i18n.gettext()

ITER_NONE = -1

class __TREEVIEW__ :

	"""
	The basic treeview class.

	Properties
	===========
		enable-search
		expander-column
		fixed-height-mode
		hadjustment
		headers-clickable
		headers-visible
		hover-expand
		hover-selection
		model
		reorderable
		rules-hint
		search-column
		vadjustment

	PROPERTIES = { \
		"enable-search" : True, \
		"expander-column" : , \
		"fixed-height-mode" : True, \
		"hadjustment" : True, \
		"headers-clickable" : True, \
		"headers-visible" : True, \
		"hover-expand" : True, \
		"hover-selection" : True, \
		"model" : True, \
		"reorderable" : False, \
		"rules-hint" : True, \
		"search-column" : -1, \
		"vadjustment" : , \
	}

	"""

	XML = None
	WIDGET_TREEVIEW = None
	WIDGET_TREESTORE = None
	MODEL = None

	"""
	GTK selection constants are,
		gtk.SELECTION_NONE	No selection allowed.
		gtk.SELECTION_SINGLE	A single selection allowed by clicking.
		gtk.SELECTION_BROWSE	A single selection allowed by browsing with the pointer.
		gtk.SELECTION_MULTIPLE	Multiple items can be selected at once.
		gtk.SELECTION_EXTENDED	Deprecated.
	"""
	SELECTION_MODE = gtk.SELECTION_SINGLE
	SELECTION_CURRENT = None

	def __init__ (self, xml, treeview_name, properties=dict()) :

		self.XML = xml
		self.WIDGET_TREEVIEW = self.XML.get_widget(treeview_name)

		# set property
		if len(properties) > 0 :
			for i, j in properties.items() :
				self.WIDGET_TREEVIEW.set_property(i, j)

		self.create_frame()
		self.insert_row()

		__selection = self.WIDGET_TREEVIEW.get_selection()
		__selection.set_mode(self.SELECTION_MODE)
		__selection.connect("changed", self.on_change_selection)

		# connect handler
		self.WIDGET_TREEVIEW.connect("row-activated", self.on_row_activated)

	def create_frame (self) : pass
	def insert_row (self) : pass
	def add_row (self, *args) : pass

	###############################################################
	# misc methods
	def prepend (self, rows) :
		self.WIDGET_TREESTORE.prepend(None, rows)

	def set_cursor (self, path) :
		self.WIDGET_TREEVIEW.set_cursor(path)

	def move (self, from_path, to_path) :
		if from_path == to_path : return

		__iter_from = self.WIDGET_TREESTORE.get_iter(from_path)
		if not __iter_from : return
		__iter_to = self.WIDGET_TREESTORE.get_iter(to_path)
		if not __iter_to : return

		self.WIDGET_TREESTORE.move_after(__iter_from, __iter_to)

	def remove (self, **kwargs) :
		"""
		One of path, iter must be given.

		@return : bool()
		"""

		treeiter = None
		if kwargs.has_key("treeiter") : treeiter = kwargs["treeiter"]
		if kwargs.has_key("path") :
			treeiter = self.WIDGET_TREESTORE.get_iter(kwargs["path"])

		if not treeiter : return False

		self.WIDGET_TREESTORE.remove(treeiter)
		self.WIDGET_TREEVIEW.set_cursor((1,))

	def get_path (self, iter) :
		return self.WIDGET_TREESTORE.get_path(iter)

	def get_iter (self, path) :
		return self.WIDGET_TREESTORE.get_iter(path)

	def get_value (self, path, column_number) :
		__treeiter = self.WIDGET_TREESTORE.get_iter(path)
		return self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

	def set_value (self, path, column_number, value) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)

		if not __iter : return False

		self.WIDGET_TREESTORE.set_value( \
			__iter, \
			column_number, \
			value \
		)

	def get_selected (self) :
		(__treestore, __treeiter) = \
			self.WIDGET_TREEVIEW.get_selection().get_selected()

		return (__treestore, __treeiter, )

	def get_value_selected (self, column_number) :
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter : return

		return self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

	################################################################
	# HANDLER
	def on_change_selection (self, selection) :
		self.SELECTION_CURRENT = selection

	def on_row_activated \
			(self, treeview, path, view_column, user_param1=None) :
		pass

class Diff (__TREEVIEW__) :

	STATE = None
	GCONF_CLIENT = None

	NOTE = None

	def __init__ (self, xml, **kwargs) :

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]

		if kwargs.has_key("note") :
			self.NOTE = kwargs["note"]

		PROPERTIES = { \
			"enable-search" : False, \
			"fixed-height-mode" : False, \
			"headers-clickable" : True, \
			"headers-visible" : True, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : True, \
			"rules-hint" : True, \
		}

		self.GCONF_CLIENT = GConf.Client()
		self.GCONF_CLIENT.set_path(Conf.LOCAL_ROOT)
		self.GCONF_CLIENT.add_dir(Conf.LOCAL_ROOT)

		__TREEVIEW__.__init__( \
			self, xml, "treeview_diff", properties=PROPERTIES)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererText()
		__cell1 = gtk.CellRendererText()

		# set property.
		__cell0.set_property("xalign", 0.5)

		self.tvcolumn0 = gtk.TreeViewColumn(_("Changed Time"))
		self.tvcolumn1 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)

		self.WIDGET_TREESTORE = gtk.TreeStore(str, str)
		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)

		# add the cells to the columns - 2 in the first
		self.tvcolumn0.pack_start(__cell0, False)
		self.tvcolumn1.pack_start(__cell1, False)

		self.tvcolumn0.set_attributes( \
			__cell0, text=Conf.TREEVIEW_DIFF_PANEL_COLUMN_NAME)
		self.tvcolumn1.set_attributes( \
			__cell1, text=Conf.TREEVIEW_DIFF_PANEL_COLUMN_TIME)

		self.tvcolumn0.set_resizable(True)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

		self.tvcolumn0.set_reorderable(True)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()

		if not self.NOTE : return

		for i, j in self.NOTE.getListDiff() :
			self.add_row(i)

	def add_row (self, time_to_be_diff) :
		__time = WEE.format_timestamp(self.GCONF_CLIENT, int(time_to_be_diff))

		self.WIDGET_TREESTORE.append(None, [__time, time_to_be_diff, ])

	def prepend (self, time_to_be_diff) :
		__time = WEE.format_timestamp(self.GCONF_CLIENT, int(time_to_be_diff))

		self.WIDGET_TREESTORE.prepend(None, [__time, time_to_be_diff, ])

class Attachment (__TREEVIEW__) :

	STATE = None
	GCONF_CLIENT = None

	ATTACHMENT = None
	GNOME_ICON = None

	DEFAULT_ICON = None

	def __init__ (self, xml, **kwargs) :
		"""
		kwargs is,
			state
		"""

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]

		if kwargs.has_key("attachment") :
			self.ATTACHMENT = kwargs["attachment"]

		self.GNOME_ICON = GNOME.Icon()

		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : True, \
			"headers-visible" : True, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : True, \
			"rules-hint" : True, \
		}

		self.GCONF_CLIENT = GConf.Client()
		self.GCONF_CLIENT.set_path(Conf.LOCAL_ROOT)
		self.GCONF_CLIENT.add_dir(Conf.LOCAL_ROOT)

		__TREEVIEW__.__init__( \
			self, xml, "treeview_attachment", properties=PROPERTIES)

		self.set_cursor((0,))

	def set_attachment (self, attachment) :
		if not isinstance(attachment, AttachmentCore.Core) :
			raise ValueError, "attachment must be the instance of AttachmentCore."
		else :
			self.ATTACHMENT = attachment

		self.insert_row()

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()
		__cell3 = gtk.CellRendererText()

		# set property.
		__cell1.set_property("width", 50)
		__cell1.set_property("xalign", 0.5)
		__cell2.set_property("xalign", 0.0)
		__cell3.set_property("xalign", 0.0)

		self.tvcolumn1 = gtk.TreeViewColumn()
		self.tvcolumn2 = gtk.TreeViewColumn(_("Name"))
		self.tvcolumn3 = gtk.TreeViewColumn(_("Upload Date"))
		self.tvcolumn2.set_expand(True)

		self.WIDGET_TREESTORE = gtk.TreeStore(gtk.gdk.Pixbuf, str, str)
		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn2)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn3)

		# add the cells to the columns - 2 in the first
		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn2.pack_start(__cell2, False)
		self.tvcolumn3.pack_start(__cell3, False)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=Conf.TREEVIEW_ATTACHMENT_PANEL_COLUMN_ICON)
		self.tvcolumn2.set_attributes( \
			__cell2, text=Conf.TREEVIEW_ATTACHMENT_PANEL_COLUMN_FILENAME)
		self.tvcolumn3.set_attributes( \
			__cell3, text=Conf.TREEVIEW_ATTACHMENT_PANEL_COLUMN_DATE)

		self.tvcolumn2.set_resizable(True)
		self.tvcolumn3.set_resizable(True)
		self.tvcolumn2.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn3.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

		self.tvcolumn2.set_reorderable(True)
		self.tvcolumn3.set_reorderable(True)

		# get gnome default icon
		__pixbuf = self.GNOME_ICON.getDefaultPixbuf()
		if not __pixbuf :
			self.DEFAULT_ICON = self.GNOME_ICON.getNone()
		else :
			self.DEFAULT_ICON = __pixbuf

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()

		if not self.ATTACHMENT : return

		for i in self.ATTACHMENT.get_list() :
			self.add_row(i)

	def add_row (self, filename) :

		gnome_vfs_info = self.ATTACHMENT.get_info(filename)

		# get icon
		__pixbuf = self.GNOME_ICON.getPixbufByFilename( \
			self.ATTACHMENT.get_full_path(filename))
		if not __pixbuf : __pixbuf = self.DEFAULT_ICON

		# get ctime
		__time = WEE.format_timestamp(self.GCONF_CLIENT, int(gnome_vfs_info.ctime))

		self.WIDGET_TREESTORE.append(None, [__pixbuf , filename, __time, ])

	def prepend (self, filename) :
		gnome_vfs_info = self.ATTACHMENT.get_info(filename)

		# get icon
		__pixbuf = self.GNOME_ICON.getPixbufByFilename( \
			self.ATTACHMENT.get_full_path(filename))
		if not __pixbuf : __pixbuf = self.DEFAULT_ICON

		# get ctime
		__time = WEE.format_timestamp(self.GCONF_CLIENT, int(gnome_vfs_info.ctime))

		self.WIDGET_TREESTORE.prepend(None, [__pixbuf , filename, __time, ])

class Label (WEE.Core) :

	XML = None
	WIDGET = None
	MODEL = None
	LABEL = None
	GCONF_CLIENT = None

	CALLBACK_FUNCTIONS = dict()
	STATE = None

	LABEL_COLOR_HEIGHT = None

	def __init__ (self, xml, **kwargs) :

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			self.LABEL = self.STATE.get_value("label")

		self.XML = xml
		self.WIDGET = self.XML.get_widget("treeview_label")

		self.GCONF_CLIENT = GConf.Client()
		self.GCONF_CLIENT.set_path(Conf.LOCAL_ROOT)
		self.GCONF_CLIENT.add_dir(Conf.LOCAL_ROOT)

		self.__create_frame()
		self.insert_data()

	def __create_frame (self) :
		self.WIDGET.set_headers_visible(True)

		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell1.set_property("width", Conf.PREFERENCES_LABEL_SIZE[0] + 5)
		__cell1.set_property("xalign", 0.5)
		__cell2 = gtk.CellRendererText()
		__cell3 = gtk.CellRendererText()

		self.tvcolumn1 = gtk.TreeViewColumn(_("Color"))
		self.tvcolumn2 = gtk.TreeViewColumn(_("Name"))
		self.tvcolumn3 = gtk.TreeViewColumn()
		#self.tvcolumn2.set_expand(True)

		self.TREESTORE = gtk.TreeStore(gtk.gdk.Pixbuf, str, str)
		self.WIDGET.set_model(self.TREESTORE)

		# add columns to treeview
		self.WIDGET.append_column(self.tvcolumn1)
		self.WIDGET.append_column(self.tvcolumn2)

		# add the cells to the columns - 2 in the first
		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn2.pack_start(__cell2, False)
		self.tvcolumn3.pack_start(__cell3, False)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=Conf.TREEVIEW_LABEL_PANEL_COLUMN_COLOR)
		self.tvcolumn2.set_attributes( \
			__cell2, text=Conf.TREEVIEW_LABEL_PANEL_COLUMN_NAME)
		self.tvcolumn3.set_attributes( \
			__cell3, text=Conf.TREEVIEW_LABEL_PANEL_COLUMN_UID)

		self.tvcolumn1.set_resizable(True)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

		self.tvcolumn1.set_reorderable(True)

	def insert_data (self) :
		self.TREESTORE.clear()

		for i in self.LABEL.ITEM_LIST.keys() :
			__data = self.LABEL.get_value(i, "asDict")
			if not __data : continue

			if __data["is_default"] :
				__name = "%s (%s)" % (__data["name"], _("Default"), )
			else :
				__name = __data["name"]

			self.add_item( \
				i, \
				__name, \
				__data["color"] \
			)

	def add_item (self, uid, name, color) :
		__pixbuf = func.get_pixbuf_filled_with_color(color, Conf.PREFERENCES_LABEL_SIZE)

		self.TREESTORE.append(None, [__pixbuf , name, uid, ])

	def __get_iter (self, path) :
		__treeiter = self.TREESTORE.get_iter(path)
		return __treeiter

	def get_value (self, path, column_number) :
		__treeiter = self.TREESTORE.get_iter(path)
		__value = self.TREESTORE.get_value(__treeiter, column_number)

		return __value

class NoteList (WEE.Core) :

	XML = None
	WIDGET = None
	MODEL = None
	NOTE = None
	LABEL = None
	GCONF_CLIENT = None

	STATE = None

	TIMEOUT_ELAPSED_TIME = None
	TEXT_WRITE_NEW = \
		"<span size=\"xx-large\" weight=\"bold\" foreground=\"grey\">%s</span>" % \
			_("Write New")

	IS_REMOVED = False
	TEXT_REMOVED = \
		"<span size=\"xx-large\" weight=\"bold\" foreground=\"grey\">%s</span>" % \
			_("Removed Notes")

	IS_SEARCH_RESULT = False
	TEXT_SEARCH_RESULT = \
		"<span size=\"xx-large\" weight=\"bold\" foreground=\"grey\">%s</span>" % \
			_("Search Result")

	SELECTION_MODE = gtk.SELECTION_SINGLE
	CELL_BLANK = None

	def __init__ (self, xml, **kwargs) :

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			self.NOTE = self.STATE.get_value("note")
			self.REMOVED = self.STATE.get_value("note-removed")
			self.LABEL = self.STATE.get_value("label")

		self.XML = xml
		self.WIDGET = self.XML.get_widget("treeview_note")

		__selection = self.WIDGET.get_selection()
		__selection.set_mode(self.SELECTION_MODE)
		__selection.connect("changed", self.on_change_selection)

		WEE.Core.__init__(self)

		for __signal, __handler in Handler.PANEL_NOTE.items() :
			self.WIDGET.connect(__signal, eval(__handler))

		self.GCONF_CLIENT = GConf.Client()
		self.GCONF_CLIENT.set_path(Conf.LOCAL_ROOT)
		self.GCONF_CLIENT.add_dir(Conf.LOCAL_ROOT)

		self.EVENTBOX = self.XML.get_widget("eventbox4")
		self.EVENTBOX.connect("key-release-event", self.on_key_press)
		self.EVENTBOX.connect("key-press-event", self.on_key_press)

		self.add_timeout_elapsed_time()

		self.NOT_UPDATE_FRAME_ITEM = True
		self.__create_frame()

		# check whether the finished note will be viewed or not.
		__initial_filter_condition = dict()
		__initial_filter_condition["view_finished"] = \
			self.GCONF_CLIENT.get_value_by_conf("view-finished")

		self.insert_data(**__initial_filter_condition)
		self.NOT_UPDATE_FRAME_ITEM = False

		# set the cursor to the (1,) as default.
		self.WIDGET.set_cursor((1,))

	def on_change_selection (self, selection) :
		#selection.get_selected_rows()
		pass

	def __create_frame (self) :
		self.WIDGET.set_headers_visible(False)

		# create a CellRenderers to render the data
		cell0 = gtk.CellRendererPixbuf()
		cell0.set_property("xalign", 1.0)
		cell0.set_property("yalign", 0.0)
		cell1 = gtk.CellRendererText()
		cell2 = gtk.CellRendererText()

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn1 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)
		self.tvcolumn1.set_max_width(1)

		# add the cells to the columns - 2 in the first
		self.tvcolumn0.pack_start(cell0, False)
		self.tvcolumn0.pack_start(cell1, False)
		self.tvcolumn1.pack_start(cell2, False)

		self.TREESTORE = gtk.TreeStore(gtk.gdk.Pixbuf, str, str)
		self.STATE.set_value("note-treestore", self.TREESTORE)
		self.WIDGET.set_model(self.TREESTORE)

		self.WIDGET.append_column(self.tvcolumn0)

		self.tvcolumn0.set_attributes(cell0, pixbuf=0)
		self.tvcolumn0.set_attributes(cell1, markup=1)
		self.tvcolumn1.set_attributes(cell2, text=2)

		self.tvcolumn0.set_sort_column_id(Conf.TREEVIEW_NOTE_PANEL_COLUMN_TEXT)

		#self.tvcolumn1.set_resizable(True)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn0.set_reorderable(True)

		# make treeview searchable
		self.WIDGET.set_search_column(Conf.TREEVIEW_NOTE_PANEL_COLUMN_TEXT)
		self.WIDGET.set_reorderable(True)
		self.WIDGET.set_expander_column(self.tvcolumn0)

	UID_AND_PATH = dict()
	def getPathByUID (self, uid) :
		self.UID_AND_PATH = dict()
		self.TREESTORE.foreach(self.__getPathByUID, uid)
		if self.UID_AND_PATH.has_key(uid) :
			path = self.UID_AND_PATH[uid]
		else :
			path = None

		return path

	def __getPathByUID (self, model, path, iter, uid) :
		__uid = self.get_value(path, Conf.TREEVIEW_NOTE_PANEL_COLUMN_UID)
		self.UID_AND_PATH[__uid] = path

	LIST_BY_PARENT = dict()
	def insert_data (self, **kwargs) :
		self.TREESTORE.clear()

		__pixbuf = func.get_pixbuf_filled_with_color(None, Conf.NOTE_LABEL_SIZE)

		##################################################### view REMOVED.
		if kwargs.has_key("uid_label") and \
				kwargs["uid_label"] == "__view_removed__" :

			self.IS_REMOVED = True
			__uid_list = self.REMOVED.get_list()
			__obj_list = self.REMOVED
			self.ROW_CREATE_NEW = \
				self.TREESTORE.insert( \
					None, 0, [__pixbuf, self.TEXT_REMOVED, "", ])

			# insert data into treeview.
			for i in __uid_list :
				__data = __obj_list.get_value(i, "asDict")

				if not __data : continue
				if self.__filter(__data, **kwargs) : continue

				self.add_item( \
					i, \
					func.unquote_filename(__data["title"]), \
					__data["__timestamp_last_modified__"], \
					__data["__timestamp_finished__"], \
					__data["__timestamp_added__"], \
					__data["label"], \
					False, \
				)
		################################################# view NOT REMOVED.
		else :
			self.LIST_BY_PARENT = self.NOTE.getListByParent()
			self.ROW_CREATE_NEW = \
				self.TREESTORE.insert( \
					None, 0, [__pixbuf, self.TEXT_WRITE_NEW, "", ])

			# insert data.
			self.__insertDataByParent(None, NotesItem.UID_NO_PARENT, **kwargs)

		self.WIDGET.set_data("row-create-new", self.ROW_CREATE_NEW)

		"""
		Sort item by __timestamp_last_modified__ and __timestamp_finished__.
		"""

	def __insertDataByParent (self, iter_parent, uid_parent, **kwargs) :

		if not hasattr(self, "LIST_BY_PARENT") : return ITER_NONE
		if not self.LIST_BY_PARENT.has_key(uid_parent) : return ITER_NONE

		(__list_uid_active, __list_uid_finished, ) = \
			self.NOTE.sort(self.LIST_BY_PARENT, uid_parent)

		if not self.GCONF_CLIENT.get_value_by_conf("view-finished") :
			__list_uid = __list_uid_active
		else :
			__list_uid = __list_uid_active + __list_uid_finished

		for i in __list_uid :
			__data = self.NOTE.get_value(i, "asDict")

			if not __data : continue
			if self.__filter(__data, **kwargs) : continue

			__iter = self.append_item( \
				iter_parent, \
				i, \
				func.unquote_filename(__data["title"]), \
				__data["__timestamp_last_modified__"], \
				__data["__timestamp_finished__"], \
				__data["__timestamp_added__"], \
				__data["label"], \
			)

			if __iter != ITER_NONE :
				self.__insertDataByParent(__iter, i, **kwargs)

	def insert_search_result (self, uid_list) :

		self.TREESTORE.clear()

		__pixbuf = func.get_pixbuf_filled_with_color(None, Conf.NOTE_LABEL_SIZE)
		self.ROW_CREATE_NEW = \
			self.TREESTORE.insert( \
				None, 0, [__pixbuf, self.TEXT_SEARCH_RESULT, "", ])
		self.WIDGET.set_data("row-create-new", self.ROW_CREATE_NEW)

		# insert data into treeview.
		for i in uid_list :
			__data = self.NOTE.get_value(i, "asDict")
			if not __data : continue

			#if self.__filter(__data, **kwargs) : continue

			self.add_item( \
				i, \
				func.unquote_filename(__data["title"]), \
				__data["__timestamp_last_modified__"], \
				__data["__timestamp_finished__"], \
				__data["__timestamp_added__"], \
				__data["label"], \
				False, \
			)

	def __filter (self, data_dict, **kwargs) :

		# check uid
		if kwargs.has_key("uid_label") :
			if kwargs["uid_label"] in ["__view_removed__", ] :
				return False

			if kwargs["uid_label"] not in ["__view_all__", "__view_removed__", ] :
				if data_dict["label"] != kwargs["uid_label"] :
					return True

		# check the finished view
		if kwargs.has_key("view_finished") :
			if not kwargs["view_finished"] :
				if data_dict["__timestamp_finished__"] > 0 :
					return True

		return False

	def append_item \
		( \
			self, \
			iter_parent, \
			uid, title, \
			modified_time, finished_time, added_time, \
			uid_label \
		) :

		if finished_time > 0 :
			__string = WEE.note_format_string_finished( \
				self.GCONF_CLIENT, title, finished_time)
		else :
			__string = \
				WEE.note_format_string(self.GCONF_CLIENT, title, modified_time, added_time)

		try :
			__color = self.LABEL.get_value(uid_label, "asDict")["color"]
		except TypeError : # unknown label
			__color = None

		__pixbuf = func.get_pixbuf_filled_with_color(__color, Conf.NOTE_LABEL_SIZE)

		__iter = self.TREESTORE.append(iter_parent, [__pixbuf, __string , uid, ])

		return __iter

	def add_item \
		( \
			self, uid, title, \
			modified_time, finished_time, added_time, \
			uid_label, \
			append=True \
		) :

		if finished_time > 0 :
			__string = WEE.note_format_string_finished( \
				self.GCONF_CLIENT, title, finished_time)
		else :
			__string = \
				WEE.note_format_string(self.GCONF_CLIENT, title, modified_time, added_time)

		try :
			__color = self.LABEL.get_value(uid_label, "asDict")["color"]
		except TypeError : # unknown label
			__color = None

		__pixbuf = func.get_pixbuf_filled_with_color(__color, Conf.NOTE_LABEL_SIZE)

		if not append :
			__iter = self.TREESTORE.insert_after( \
				None, self.ROW_CREATE_NEW, [__pixbuf, __string , uid, ])
		else :
			__iter = self.TREESTORE.append(None, [__pixbuf, __string , uid, ])

		return __iter

	def __get_iter (self, path) :
		__treeiter = self.TREESTORE.get_iter(path)
		return __treeiter

	def get_value (self, path, column_number) :
		__treeiter = self.TREESTORE.get_iter(path)
		__value = self.TREESTORE.get_value(__treeiter, column_number)

		return __value

	################################################################
	# HANDLER
	def add_timeout_elapsed_time (self) :
		if self.TIMEOUT_ELAPSED_TIME : return

		#timeout = 1000 * 3  # milliseconds, 1000 is 1 second.
		#interval = 1000  # milliseconds, 1000 is 1 second.
		interval = self.GCONF_CLIENT.get_value_by_conf("note-list-update-interval")

		self.TIMEOUT_ELAPSED_TIME = \
			gobject.timeout_add(interval, self.update_elapsed_time)

	def remove_timeout_elapsed_time (self) :
		if self.TIMEOUT_ELAPSED_TIME :
			gobject.source_remove(self.TIMEOUT_ELAPSED_TIME)

		self.TIMEOUT_ELAPSED_TIME = None

	def update_elapsed_time (self) :
		self.TREESTORE.foreach(self.__update_elapsed_time)
		return True

	def __update_elapsed_time (self, model, path, iter) :
		if path == str(Conf.TREEVIEW_NOTE_PATH_CREATE_NEW[0]) : return

		__uid = self.get_value(path, Conf.TREEVIEW_NOTE_PANEL_COLUMN_UID)
		if not __uid.strip() : return

		__data = self.NOTE.get_value(__uid, "asDict")

		if not __data : return
		if __data["__timestamp_finished__"] > 0 : return

		__string = WEE.note_format_string( \
			self.GCONF_CLIENT, \
			func.unquote_filename(__data["title"]), \
			__data["__timestamp_last_modified__"], \
			__data["__timestamp_added__"] \
		)

		self.TREESTORE[path][Conf.TREEVIEW_NOTE_PANEL_COLUMN_TEXT] = __string

	def on_row_activated \
			(self, treeview, path, view_column, user_param1=None) :

		__uid = self.get_value(path, Conf.TREEVIEW_NOTE_PANEL_COLUMN_UID)
		if not __uid.strip() : # add new.
			if self.IS_SEARCH_RESULT : return
			if self.IS_REMOVED : return

			__uid = func.get_uid()
			view = Conf.NOTE_NOTEBOOK_TAB_RAW
		else :
			view = Conf.NOTE_NOTEBOOK_TAB_CONTENT

		if self.IS_REMOVED :
			Window.Note(__uid, is_removed=True, view=view, state=self.STATE)
		else :
			Window.Note(__uid, view=view, state=self.STATE)

		return

	def on_key_press (self, widget, event) :

		if event.type in [gtk.gdk.KEY_RELEASE, gtk.gdk.KEY_PRESS, ] :
		 	__keyname = func.get_keyname(event, ignore_shift=True)
			if __keyname not in Conf.NOTELIST_PANEL_TREEVIEW_KEYNAME_AVAILABLE :
				return

			(__treestore, __treeiter) = \
				self.WIDGET.get_selection().get_selected()

			if not __treeiter : return

			__path = __treestore.get_path(__treeiter)
			uid = __treestore.get_value(__treeiter, \
				Conf.TREEVIEW_NOTE_PANEL_COLUMN_UID)

		if event.type == gtk.gdk.KEY_RELEASE :

			if __keyname in ["Right", "l", ] : # expand
				self.WIDGET.expand_row(__path, False)
				return

			elif __keyname in ["Left", "h", ] : # collapse
				if not self.WIDGET.row_expanded(__path) :
					__iter_parent = __treestore.iter_parent(__treeiter)
					if not __iter_parent : return
					self.WIDGET.set_cursor(__treestore.get_path(__iter_parent))
					return
				else :
					self.WIDGET.collapse_row(__path)
				return

			elif __keyname.lower() == "v" : # view
				Window.Note( \
					uid, is_removed=self.IS_REMOVED, view=Conf.NOTE_NOTEBOOK_TAB_CONTENT, state=self.STATE)

			elif __keyname.lower() == "e" : # edit
				Window.Note( \
					uid, is_removed=self.IS_REMOVED, view=Conf.NOTE_NOTEBOOK_TAB_RAW, state=self.STATE)

			elif __keyname.lower() == "a" : # alarm
				Window.Note( \
					uid, is_removed=self.IS_REMOVED, view=Conf.NOTE_NOTEBOOK_TAB_SETTING, state=self.STATE)

			elif __keyname.lower() == "i" : # information
				Window.Note( \
					uid, is_removed=self.IS_REMOVED, view=Conf.NOTE_NOTEBOOK_TAB_INFO, state=self.STATE)

			elif __keyname.lower() == "n" : # create new note.
				if self.IS_REMOVED : return

				Window.Note(func.get_uid(), state=self.STATE)

			#elif __keyname in ["Delete", "r"] :
			#	if not self.NOTE.remove(uid) : return
			#	self.TREESTORE.remove(__treeiter)
			#	self.NOTE.__export__()

			elif __keyname in ["s", ] : # toggle finish/re-entry
				Window.Note( \
					func.get_uid(), \
					is_removed=self.IS_REMOVED, \
					parent=uid, \
					view=Conf.NOTE_NOTEBOOK_TAB_RAW, \
					state=self.STATE \
				)
			elif __keyname in ["Delete", "r", ] : # toggle finish/re-entry
				__value = self.NOTE.get_value(uid, "asDict")

				if int(__value["__timestamp_finished__"]) > 0 :
					was_finish = True
				else :
					was_finish = False

				self.NOTE.set_finish(uid, not was_finish)
				self.NOTE.__export__()

				self.toggle_finish(uid, __treeiter, not was_finish)

		elif event.type == gtk.gdk.KEY_PRESS :
			if __keyname == "j" : # move down the cursor

				#if __treestore.iter_has_child(__treeiter) :
					#self.WIDGET.expand_row(__path, False)
				if self.WIDGET.row_expanded(__path) :
					__iter_next = self.TREESTORE.iter_children(__treeiter)
				else :
					__iter_next = self.TREESTORE.iter_next(__treeiter)
					if not __iter_next :
						if len(__path) == 1 : return
						__iter_parent = __treeiter
						while not __iter_next :
							__iter_parent = self.TREESTORE.iter_parent(__iter_parent)
							if not __iter_parent : return
							__iter_next = self.TREESTORE.iter_next(__iter_parent)

				self.WIDGET.set_cursor(self.TREESTORE.get_path(__iter_next))
				return

			elif __keyname == "k" : # move up the cursor
				__path0 = list(__path)

				# get previous path
				__pos = __path0[-1] - 1

				__path1 = __path0[:-1]
				if __pos < 0 : # go parent
					if len(__path0) == 1 : return
				else : # go previous
					__path1.append(__pos)

				__path1 = tuple(__path1)
				# check the previous path is valid or not.
				if __treestore.get_iter(__path1) :
					__path_previous = __path1
				else :
				# if path is not valid, get parent iter
					__path_previous = \
						__treestore.get_path(__treestore.iter_parent(__treeiter))

				self.WIDGET.set_cursor(__path_previous)
				return

		return

	def append_finished_row (self, **kwargs) :

		__filter_condition = kwargs
		__filter_condition["view_finished"] = True

		# get list of the finished notes.
		__inserting_notes_uid = list()
		for i, j in self.NOTE.ITEM_LIST.items() :
			if j.get_value("__timestamp_finished__") > 0 :
				__inserting_notes_uid.append(i)

		# append in treestore.
		for i in __inserting_notes_uid :
			__data = self.NOTE.get_value(i, "asDict")
			if not __data : continue

			if self.__filter(__data, **__filter_condition) : continue

			self.add_item( \
				i, \
				func.unquote_filename(__data["title"]), \
				__data["__timestamp_last_modified__"], \
				__data["__timestamp_finished__"], \
				__data["__timestamp_added__"], \
				__data["label"], \
				True
			)

	__REMOVING_ITER = list()
	def remove_finished_row (self) :
		self.__REMOVING_ITER = list()

		self.TREESTORE.foreach(self.__remove_finished_row)

		[self.remove(treeiter=i) for i in self.__REMOVING_ITER]

	def __remove_finished_row (self, model, path, iter) :
		if self.NOTE.get_value_by_key( \
				model.get_value(iter, Conf.TREEVIEW_NOTE_PANEL_COLUMN_UID), \
				"__timestamp_finished__") > 0 :
			self.__REMOVING_ITER.append(iter)

		return

	def __set_finish (self, data, treeiter) :
		"""
		Finish this note.

		When the note finish,
			* store the finished time and modified time (current time) as second.
			* move item to the last position in the panel.
		"""
		# remove it first from panel.
		self.remove(treeiter=treeiter)

		if not self.GCONF_CLIENT.get_value_by_conf("view-finished") :
			return

		__this_time = int(time.time())

		# append this in panel.
		return self.add_item( \
			data["__uid__"], \
			func.unquote_filename(data["title"]), \
			data["__timestamp_last_modified__"], \
			data["__timestamp_finished__"], \
			data["__timestamp_added__"], \
			data["label"], \
			True, \
		)

	def __unset_finish (self, data, treeiter) :
		"""
		Un-Finish this note.

		When the note un-finish,
			* set the finished time to 0.
			* set the modified time to current time.
			* move item to the top position in the panel.
		"""

		if treeiter : self.remove(treeiter=treeiter)

		# append this in panel.
		return self.add_item( \
			data["__uid__"], \
			func.unquote_filename(data["title"]), \
			data["__timestamp_last_modified__"], \
			data["__timestamp_finished__"], \
			data["__timestamp_added__"], \
			data["label"], \
			False, \
		)

	def toggle_finish (self, uid, treeiter, isFinish=True) :
		if treeiter :
			(__uid, __data) = self.__get_value_by_iter(treeiter)
		else :
			(__uid, __data) = (uid, self.NOTE.get_value(uid, "asDidct"))

		if isFinish : # it's already finished
			retval = self.__set_finish(__data, treeiter)
		else : # not yet finished
			retval = self.__unset_finish(__data, treeiter)

		self.WIDGET.set_cursor((1,))

		return retval

	def __get_value_by_iter (self, treeiter) :

		# get uid and it's data.
		__uid = self.get_value( \
			self.TREESTORE.get_path(treeiter), \
			Conf.TREEVIEW_NOTE_PANEL_COLUMN_UID, \
		)

		return (__uid, self.NOTE.get_value(__uid, "asDict"), )

	def move (self, from_path, to_path) :
		if from_path == to_path : return

		__iter_from = self.TREESTORE.get_iter(from_path)
		if not __iter_from : return
		__iter_to = self.TREESTORE.get_iter(to_path)
		if not __iter_to : return

		self.TREESTORE.move_after(__iter_from, __iter_to)

	def remove (self, **kwargs) :
		"""
		One of path, iter must be given.

		@return : bool()
		"""

		treeiter = None
		if kwargs.has_key("treeiter") : treeiter = kwargs["treeiter"]
		if kwargs.has_key("path") :
			treeiter = self.TREESTORE.get_iter(kwargs["path"])

		if not treeiter : return False

		self.TREESTORE.remove(treeiter)
		self.WIDGET.set_cursor((1,))

	def set_value (self, path, column_number, value) :
		__iter = self.TREESTORE.get_iter(path)

		if not __iter : return False

		self.TREESTORE.set_value( \
			__iter, \
			column_number, \
			value \
		)


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Panel.py"


