# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome.applet, gobject
import os
import Alarm, Conf, Conf__, i18n, Window, State, func, \
	NotesItem, LabelItem, RemovedItems

_ = i18n.gettext()

class Core :

	STATE = None

	def __init__ (self, **kwargs) :

		check_environment()

		if kwargs.has_key("state") :
			self.STATE = kwargs["state"]
		else :
			self.STATE = State.Main()

class Applet (gnome.applet.Applet, Core) :

	NOTE = None
	LABEL = None

	applet = None

	def __init__ (self, applet, iid) :

		self.__gobject_init__()
		Core.__init__(self)

		self.PROPXML_BUTTON_3 = \
"""
<popup name="button3">
	<menuitem name="Item 1" verb="AddNote" label="%(addnote)s" pixtype="stock" pixname="gtk-properties"/>
	<menuitem name="Item 2" verb="Prefs" label="%(prefs)s" pixtype="stock" pixname="gtk-properties"/>
	<menuitem name="Item 3" verb="About" label="%(about)s" pixtype="stock" pixname="gnome-stock-about"/>
</popup>
""" % { \
	"addnote" : _("_Add Note"),
	"prefs" : _("_Preferences"),
	"about" : _("About"), \
}

		self.VERBS = [ \
			("AddNote", self.on_add_note ), \
			("List", self.on_list ), \
			("Prefs", self.on_preferences ), \
			("About", self.on_about ), \
		]

		self.applet = applet
		self.big_evbox = gtk.EventBox()

		__icon = gtk.Image()
		__icon.set_from_file(os.path.join(Conf__.DIR_PIXMAPS, "wee_48x48.png"))
		__icon.show()

		self.big_evbox.add(__icon)
		self.big_evbox.connect("button-press-event", self.on_button_press)
		self.big_evbox.show()

		self.applet.add(self.big_evbox)
		self.applet.show_all()

		self.add_timeout_alarm()

		# set the most important object.
		__REMOVED = RemovedItems.Them()
		__REMOVED.__import__(Conf.FILE_NOTES_REMOVED)
		self.STATE.set_value("note-removed", __REMOVED)

		__LABEL = LabelItem.Them()
		__LABEL.__import__(Conf.FILE_LABEL)
		self.STATE.set_value("label", __LABEL)

		__NOTE = NotesItem.Them(state=self.STATE)
		__NOTE.__import__(Conf.FILE_NOTES)
		self.STATE.set_value("note", __NOTE)

		self.ALARM = Alarm.Note(__NOTE, state=self.STATE)
		self.STATE.set_value("alarm", self.ALARM)

		# test
		Window.WEE(state=self.STATE)

	def add_timeout_alarm (self) :
		gobject.timeout_add(1000 * 10, self.start_alarm_clock)

	def start_alarm_clock (self) :
		self.ALARM.run()

		return True

	def cleanup(self,event) :
		del self.applet

	# this callback is to create a context menu with the second button
	def create_menu (self, propxml) :
		self.applet.setup_menu(propxml, self.VERBS, None)

	##################################################
	# on method.
	def on_add_note (self, widget, event) :
		Window.Note( \
			func.get_uid(), view=Conf.NOTE_NOTEBOOK_TAB_RAW, state=self.STATE)
		return True

	def on_list (self, widget, event) :
		Window.WEE(state=self.STATE)
		return True

	def on_button_press (self, widget, event) :
		if event.type == gtk.gdk.BUTTON_PRESS :
			if event.button == 1 :
				__widget_wee = self.STATE.get_value("widget-wee", None)
				if not __widget_wee :
					Window.WEE(state=self.STATE)
				else :
					__widget_wee.emit("delete_event", event)
					__widget_wee = self.STATE.set_value("widget-wee", None)

			elif event.button == 3 :
				self.create_menu(self.PROPXML_BUTTON_3)
			self.create_menu(self.PROPXML_BUTTON_3)

	def on_about (self, event, data=None) :
		Window.About()
		return True

	def on_preferences (self, event, data=None) :
		Window.Preferences(state=self.STATE)
		return True

def check_environment () :

	if not os.path.isdir(os.path.join(os.getenv("HOME"), Conf__.DIR_TMP)) :
		os.makedirs(os.path.join(os.getenv("HOME"), Conf__.DIR_TMP))

	if not os.path.isdir(os.path.join(os.getenv("HOME"), Conf__.DIR_LOCAL)) :
		os.makedirs(os.path.join(os.getenv("HOME"), Conf__.DIR_LOCAL))

	if not os.path.isdir(Conf.DIR_NOTE_ATTACHMENT) :
		os.makedirs(Conf.DIR_NOTE_ATTACHMENT)

	if not os.path.isdir(Conf.DIR_NOTE) :
		os.makedirs(Conf.DIR_NOTE)

	if not os.path.isdir(Conf.DIR_NOTE_REMOVED) :
		os.makedirs(Conf.DIR_NOTE_REMOVED)

####################################################################
# Functions
def format_timestamp (gconf_client, timestamp) :
	__timeformat = gconf_client.get_value_by_conf("note-time-format")

	return func.get_formatted_time(timestamp, __timeformat)

def note_format_string (gconf_client, title, event_time, added_time) :

	__timeformat = gconf_client.get_value_by_conf("note-time-format")
	__event_time = func.get_elapsed_time(event_time)
	__added_time = func.get_formatted_time(added_time, __timeformat)

	__format_string = \
		_("<span strikethrough=\"false\" size=\"larger\" weight=\"bold\">%s</span>\n<span size=\"small\"><span foreground=\"darkblue\">Since %s,</span>\n<span foreground=\"grey\">this item was added at %s</span></span>")

	__string = __format_string % (title, __event_time, __added_time)

	return __string

def note_format_string_finished (gconf_client, title, event_time) :

	__timeformat = gconf_client.get_value_by_conf("note-time-format")
	__event_time = func.get_formatted_time(event_time, __timeformat)

	__format_string = \
		_("<span foreground=\"grey\"><span strikethrough=\"false\" size=\"larger\" weight=\"bold\" >%s</span>\n<span size=\"small\">Finished at %s</span></span>")

	__string = __format_string % (title, __event_time)

	return __string


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "WEE.py"


