# XmlMarshaller v0.2
# (c) 2003 Peter Yared
# Python 2.3 License

from xml.sax import saxutils, handler, make_parser
from xml.sax.handler import feature_namespaces
from StringIO import StringIO
import datetime


# XSD Types implemented: string, integer, float, date, dateTime, time

# Special attributes for Python objects:
# __xmlname__ = Specific element name to use when marshalling the object
# __xmlnamespace__ = Namespace to use for this object
# __xmlnamespacetext__ = Embedded XSD text to use for this object
# __xmlattr__ = List of Python attributes that should be xml attributes rather than elements.  Note uses Python attribute names, even if __xmlmappingdict__ is used on attribute.
# __xmlexclude__ = List of Python attributes to exclude in marshalling.  Note uses Python attribute names, even if __xmlmappingdict__ is used on attribute.
# __xmlseqnamesdict__ = Dict of element name to use when marshalling each element of a list, ie: "addresses" : "address"
# __xmlmappingdict__ = Dict of element name to use for a particular attribute, useful for including private attributes (ones that begin with '_') that would otherwise be excluded... note that the xml element names need to be lowercase
# __xmlcomplextypename__ = A named complex type for a particular object to be used when creating an xsd for that object


# TODO: Implement boolean, finish dict
# TODO: Handle seq xsd when there is no subelement name specified, ie: list of strings
# TODO: Use feature_namespace_prefixes to remember the original prefix used for a document
# TODO: Encode unicode strings appropriately


INDENT_CHARS = '	'

class Element:

	def __init__(self, name = None):
		self.__xmlname__ = name


class ObjectCreator(handler.ContentHandler):

	def __init__(self):
		self._obj = Element(name = "Root")
		self._objectStack = [self._obj]
		self._objectSetStack = [False]
		self._nameSpaces = {}
		self._chars = ''
		self._typeMappings = {}
		self._pathMappings = {}
		self._defaultNameSpace = None
		handler.ContentHandler.__init__(self)

	def addTypeMapping(self, nameSpace, complexType, pythonClass):
		self._typeMappings[(nameSpace + '/' + comlexType).lower()] = pythonClass

	def getTypeMappings(self):
		return self._typeMappings

	def setTypeMappings(self, pathMappings):
		self._typeMappings = typeMappings

	def addPathMapping(self, path, pythonClass):
		self._pathMappings[path] = pythonClass

	def getPathMappings(self):
		return self._pathMappings

	def setPathMappings(self, pathMappings):
		self._pathMappings = pathMappings

	def getDefaultNameSpace(self):
		return self._defaultNameSpace

	def setDefaultNameSpace(self, nameSpaceName):
		self._defaultNameSpace = unmarshal(nameSpaceName)

	def setDefaultNameSpaceText(self, nameSpaceText):
		self._defaultNameSpace = unmarshalText(nameSpaceText)

	def getRootObject(self):
		return self._obj

	def startPrefixMapping(self, prefix, uri):
		pass

	def endPrefixMapping(self, prefix):
		pass


	def startElementNS(self, name, qname, attrs):
		self._chars = ''

		nameSpace = None
		nameSpaceName = name[0]
		if name[0]:
			if self._nameSpaces.has_key(name[0]):
				nameSpace = self._nameSpaces[name[0]]
			elif not name[0].lower().startswith("http://www.w3.org"):  # Don't need to parse XML Schema
				self._nameSpaces[name[0]] = unmarshal(name[0])
				nameSpace = self._nameSpaces[name[0]]
		else:
			nameSpace = self._defaultNameSpace
		name = name[1].lower()

		xsdType = None
		complexTypeName = None
		pathList = None
		if nameSpace:
			pathList = []
			for obj in self._objectStack[1:]:
				pathList.append(obj.__xmlname__)
			pathList.append(name)
			xsdType, complexTypeName = getnamespacetype(nameSpace, pathList)

		object = None

		if len(self._pathMappings) > 0:
			path = ""
			for obj in self._objectStack[1:]:
				path += obj.__xmlname__ + '.'
			path += name
			if self._pathMappings.has_key(path):
				object = self._pathMappings[path]()
				object.__xmlname__ = name

		if len(self._typeMappings) > 0 and complexTypeName and nameSpaceName:
			typeString = (nameSpaceName + '/' + complexTypeName).lower()
			if self._typeMappings.has_key(typeString):
				object = self._typeMappings[typeString]()
				object.__xmlname__ = name

		if not object:
			object = Element(name = name)
			if complexTypeName:
				object.__xmlcomplextypename__ = complexTypeName  # Remember the complex type name so it can be used if an xsd is generated from the object tree
		self._objectStack.append(object)
		self._objectSetStack.append(False)

		attributeList = []
		for (attribute, value) in attrs.items():
			attribute = attribute[1].lower()
			value = saxutils.unescape(value)
			if nameSpace:
				attrPathList = pathList[:]
				attrPathList.append(attribute)
				attrXsdType, attrComplexTypeName = getnamespacetype(nameSpace, attrPathList)
				value = _castxsdvalue(value, attrXsdType)
			try:
				if hasattr(object, "__xmlmappingdict__") and object.__xmlmappingdict__.values().index(attribute) > -1:  # attribute lookup is ok to do lowercase since it is XML element names
					attribute = object.__xmlmappingdict__.keys()[object.__xmlmappingdict__.values().index(attribute)]
			except:
				pass

			setattrignorecase(object, attribute, value)
			attributeList.append(attribute)
		if attributeList:
			setattr(object, '__xmlattr__', attributeList)


	def endElementNS(self, name, qname):
		chars = saxutils.unescape(self._chars)
		self._chars = ''

		if name[0] and not name[0].lower().startswith("http://www.w3.org"):
			nameSpace = self._nameSpaces[name[0]]
		else:
			nameSpace = self._defaultNameSpace

		name = name[1].lower()

		xsdType = None
		complexTypeName = None
		if nameSpace:
			pathList = []
			for obj in self._objectStack[1:]:
				pathList.append(obj.__xmlname__)
			xsdType, complexTypeName = getnamespacetype(nameSpace, pathList)

		object = self._objectStack.pop()
		objectSet = self._objectSetStack.pop()
		parentObject = self._objectStack[-1]

		if not xsdType and hasattr(parentObject, "__xmlseqnamesdict__") and parentObject.__xmlseqnamesdict__.has_key(name):
			xsdType = "sequence"

		try:
			if hasattr(parentObject, "__xmlmappingdict__") and parentObject.__xmlmappingdict__.values().index(name) > -1:
				name = parentObject.__xmlmappingdict__.keys()[parentObject.__xmlmappingdict__.values().index(name)]
		except:
			pass

		if chars and not objectSet:
			value = chars
			if xsdType:
				value = _castxsdvalue(value, xsdType)
			if hasattrignorecase(parentObject, name) and getattrignorecase(parentObject, name):  # If the element has already been added to the object, turn it into a list
				if isinstance(getattrignorecase(parentObject, name), type([])):
					list = getattrignorecase(parentObject, name)
					list.append(value)
					value = list
				else:
					value = [getattrignorecase(parentObject, name), value]

			setattrignorecase(parentObject, name, value)

		else:  # Doesn't have characters to add
			if isListOrTuple(parentObject):  # The parent is a list, so just add it to the list
				parentObject.append(object)
			elif hasattrignorecase(parentObject, name) and getattrignorecase(parentObject, name):  # Exists and has a value
				value = getattrignorecase(parentObject, name)
				if isListOrTuple(value):  # The Python attribute is already a list, so just add this one to the list
					value.append(object)
				else:
					setattrignorecase(parentObject, name, [value, object])  # The Python attribute already exists, so make it a list
			else:  # Add the object as a Python attribute
				if xsdType == "sequence":   # If the xsd says it's supposed to be a sequence make sure it's a list
					for (key, value) in object.__dict__.items():
						if not key.startswith("_"):
							if isListOrTuple(value):
								object = value
							else:
								if not hasattr(parentObject, "__xmlseqnamesdict__"):  # In case the sequence is composed of primitive types, remember the element name of its elements, ie for <ADDRESSES><ADDRESS/><ADDRESS/></ADDRESSES>, remember "ADDRESS"
									setattr(parentObject, "__xmlseqnamesdict__", {})
								parentObject.__xmlseqnamesdict__[object.__xmlname__] = key
								object = [value]
							break
				setattrignorecase(parentObject, name, object)
			# Check to see if this element is composed of a list of the same elements and then just put it up a level
			# For example, <PERSON><ADDRESSES><ADDRESS/><ADDRESS/><ADDRESS/></ADDRESSES></PERSON>
			# Would be accessed by person.addresses[0] instead of person.addresses.address[0]
			if xsdType != "sequence" and not isListOrTuple(object) and not hasattr(object, "__xmlattr__") and len(object.__dict__) == 2 and isListOrTuple(object.__dict__.values()[0]):
				nameToCheck = object.__dict__.values()[0][0].__xmlname__
				for objectToCheck in object.__dict__.values()[0][1:]:
					if objectToCheck.__xmlname__ != nameToCheck:
						return
				setattrignorecase(parentObject, name, object.__dict__.values()[0])

		self._objectSetStack.pop()
		self._objectSetStack.append(True)  # Note that the last object in the objectstack has been set with a value, needed to do this so that ignorable whitespace is in fact ignored


	def characters(self, content):
		self._chars += content #.strip()


def hasattrignorecase(object, name):
	for attr in dir(object):
		if attr.lower() == name.lower():
			return True
	return False


def setattrignorecase(object, name, value):
	for attr in dir(object):
		if attr.lower() == name.lower():
			setattr(object, attr, value)
			return
	setattr(object, name, value)


def getattrignorecase(object, name):
	for attr in dir(object):
		if attr.lower() == name.lower():
			return getattr(object, attr)
	return getattr(object, name)  # Throws error


def getnamespacetype(nameSpace, pathList):
	xsdType = None
	complexTypeName = None
	element = walknamespace(nameSpace, pathList)
	if element:
		if hasattr(element, "type"):
			xsdType = getattr(element, "type")
		elif hasattr(element, "sequence"):
			xsdType = "sequence"
		elif hasattr(element, "complextype"):
			nestedElement = getattr(element, "complextype")
			if hasattr(nestedElement, "name"):
				complexTypeName = getattr(nestedElement, "name")
			if hasattr(nestedElement, "sequence"):  # Sometimes complex types are just sequences: <complexType><sequence>...</sequence></complexType>
				xsdType = "sequence"
			else:
				xsdType = "complexType"
	return xsdType, complexTypeName


def walknamespace(element, pathList):

	if hasattr(element, "schema"):  # We know it's a schema, just skip the element
		return walknamespace(getattr(element, "schema"), pathList)

	if hasattr(element, "name"):  # See if it has a name that matches the pathlist
		if not element.__xmlname__ == "complextype":  # Make sure it's not a named complex type
			name = getattr(element, "name")
			if name == pathList[0]:  # It matches
				if len(pathList) == 1:
					return element  # It's the element we're looking for
				else:
					pathList = pathList[1:]  # Nest into the path list 1 item, we need to keep going...
			else:
				return None

	if isListOrTuple(element):  # If it's a list, go through each item
		for listElement in element:
			result = walknamespace(listElement, pathList)
			if result:
				return result
		return None

	if hasattr(element, "element"):
		result = walknamespace(getattr(element, "element"), pathList)
		if result:
			return result

	if hasattr(element, "attribute"):
		result = walknamespace(getattr(element, "attribute"), pathList)
		if result:
			return result

	if hasattr(element, "complextype"):
		result = walknamespace(getattr(element, "complextype"), pathList)
		if result:
			return result

	if hasattr(element, "sequence"):
		result = walknamespace(getattr(element, "sequence"), pathList)
		if result:
			return result

	return None


def unmarshal(filePath, typeMappings = None, pathMappings = None, defaultNameSpace = None, defaultNameSpaceText = None):
	parser = make_parser()
	objectCreator = ObjectCreator()
	if typeMappings:
		objectCreator.setTypeMappings(typeMappings)
	if pathMappings:
		objectCreator.setPathMappings(pathMappings)
	if defaultNameSpace:
		objectCreator.setDefaultNameSpace(defaultNameSpace)
	if defaultNameSpaceText:
		objectCreator.setDefaultNameSpaceText(defaultNameSpaceText)
	parser.setContentHandler(objectCreator)
	parser.setFeature(feature_namespaces, 1)
	parser.parse(filePath)
	return objectCreator.getRootObject()


def unmarshalText(text, typeMappings = None, pathMappings = None, defaultNameSpace = None, defaultNameSpaceText = None):
	return unmarshal(StringIO(text), typeMappings, pathMappings, defaultNameSpace, defaultNameSpaceText)


def isListOrTuple(object):
	return isinstance(object, type([])) or isinstance(object, type(()))


def isSimpleType(object):
	return isinstance(object, type('')) or isinstance(object, type(u'')) or isinstance(object, type(123)) or isinstance(object, type(123.1)) or isinstance(object, datetime.date) or isinstance(object, datetime.datetime) or isinstance(object, datetime.time)


def marshal(object, key = "", nestLevel = 0, nameSpaceAbbr = None):

	text = ''
	indent = INDENT_CHARS * nestLevel

	if not object:
		return ''

	if not key:
		if hasattr(object, "__xmlname__"):
			key = object.__xmlname__
		else:
			key = str(object.__class__).lower()

	if isListOrTuple(object):  # List, iterate through the members and marshal each one as a peer to the object's attributes
		for element in object:
			text += marshal(element, key = key, nestLevel = nestLevel, nameSpaceAbbr = nameSpaceAbbr)
		return text

	if hasattr(object, "__xmlnamespace__"):
		nameSpaceAbbr = object.__xmlabbreviatednamespace__

	if nameSpaceAbbr:
		fullNameSpaceAbbr = nameSpaceAbbr + ':'
	else:
		fullNameSpaceAbbr = ''

	text += indent + "<%s%s" % (fullNameSpaceAbbr, key)
	if hasattr(object, "__xmlattr__"):
		for attribute in object.__xmlattr__:
			if hasattr(object, "__xmlmappingdict__") and object.__xmlmappingdict__.has_key(attribute):
				attributeName = object.__xmlmappingdict__[attribute]
			else:
				attributeName = attribute
			text += " %s=%s" % (attributeName.lower(), saxutils.quoteattr(str(getattr(object, attribute))))
	if hasattr(object, "__xmlnamespace__"):
		text += " xmlns:%s=\"%s\"" % (object.__xmlabbreviatednamespace__, object.__xmlnamespace__)
	text += ">"

	if isinstance(object, type('')) or isinstance(object, type(u'')):  # String
		text += saxutils.escape(object)
	elif isinstance(object, type(123)) or isinstance(object, type(123.1)):  # Number
		text += str(object)
	elif isinstance(object, datetime.date) or isinstance(object, datetime.datetime) or isinstance(object, datetime.time):  # Datetime
		text += object.isoformat()
	elif isinstance(object, type({})):  # Dict
		for dictkey, dictvalue in object.items():
			text += INDENT_CHARS * (nestLevel + 1) + "<key>\n"
			text += marshal(dictkey, key = key, nestLevel = nestLevel + 2, nameSpaceAbbr = nameSpaceAbbr)
			text += INDENT_CHARS * (nestLevel + 1) + "</key>\n"
			text += INDENT_CHARS * (nestLevel + 1) + "<value>\n"
			text += marshal(dictvalue, key = key, nestLevel = nestLevel + 2, nameSpaceAbbr = nameSpaceAbbr)
			text += INDENT_CHARS * (nestLevel + 1) + "</value>\n"
	else:  # It's an object, nest into it
		text += '\n'
		for (attrkey, attrvalue) in object.__dict__.items():
			origattrkey = attrkey
			attrkey = attrkey.lower()

			if hasattr(object, "__xmlmappingdict__") and object.__xmlmappingdict__.has_key(origattrkey):
				attrkey = object.__xmlmappingdict__[origattrkey]

			if hasattr(object, "__xmlexclude__") and origattrkey in object.__xmlexclude__:
				continue
			elif hasattr(object, "__xmlattr__") and origattrkey in object.__xmlattr__:
				continue
			elif attrkey[0] == '_':
				continue

			if isListOrTuple(attrvalue) and hasattr(object, "__xmlseqnamesdict__") and origattrkey in object.__xmlseqnamesdict__.keys():  # It's a list, but we want each item in the list to have a special element name
				text += INDENT_CHARS * (nestLevel + 1) + "<%s%s>\n" % (fullNameSpaceAbbr, attrkey)
				text += marshal(attrvalue, key = object.__xmlseqnamesdict__[origattrkey], nestLevel = nestLevel + 2, nameSpaceAbbr = nameSpaceAbbr)
				text += INDENT_CHARS * (nestLevel + 1)  + "</%s%s>\n" % (fullNameSpaceAbbr, attrkey)
			else:
				text += marshal(attrvalue, key = attrkey.lower(), nestLevel = nestLevel + 1, nameSpaceAbbr = nameSpaceAbbr)
		text += indent
	text += "</%s%s>\n" % (fullNameSpaceAbbr, key)
	return text



def createxsd2(object=list(), key = '', subkey = '', nestLevel = 0):
	if nestLevel == 0:
		text = """<?xml version="1.0" encoding="ISO-8859-1" ?>\n<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">\n"""
	else:
		text = ''

	indent = INDENT_CHARS * nestLevel

	for __object in object :
		if not __object:
			return ''

		if not key:
			if hasattr(__object, "__xmlname__"):
				key = __object.__xmlname__
			else:
				key = str(__object.__class__).lower()

		text += indent + "<xs:element name=\"%s\"" % key
		if isSimpleType(__object):
			text += " type=\"%s\"/>\n" % _derivexsdtype(__object)
		else:
			text += '>\n'
			if isListOrTuple(__object) and len(__object) > 0:  # List
				text += INDENT_CHARS * (nestLevel + 1) + "<xs:complexType>\n"
				text += INDENT_CHARS * (nestLevel + 2) + "<xs:sequence>\n"
				if subkey:
					keytouse = subkey
				else:
					keytouse = key
				text += createxsd(__object[0], key = keytouse, nestLevel = nestLevel + 3)
				text += INDENT_CHARS * (nestLevel + 2) + "</xs:sequence>\n"
				text += INDENT_CHARS * (nestLevel + 1) + "</xs:complexType>\n"
				text += indent + "</xs:element>\n"
			elif isinstance(__object, type({})):  # Dict
				pass  # TODO
			else:  # It's an __object, nest into it
				text += INDENT_CHARS * (nestLevel + 1) + "<xs:complexType"
				if hasattr(__object, "__xmlcomplextypename__"):
					text += " name=\"%s\"" % __object.__xmlcomplextypename__
				text += '>\n'
				for (attrkey, attrvalue) in __object.__dict__.items():
					origattrkey = attrkey
					attrkey = attrkey.lower()
					if hasattr(__object, "__xmlmappingdict__") and __object.__xmlmappingdict__.has_key(origattrkey):
						attrkey = __object.__xmlmappingdict__[origattrkey]

					if hasattr(__object, "__xmlexclude__") and origattrkey in __object.__xmlexclude__:
						pass
					elif hasattr(__object, "__xmlattr__") and origattrkey in __object.__xmlattr__:
						text += INDENT_CHARS * (nestLevel + 2) + "<xs:attribute name=\"%s\" type=\"%s\"/>\n" % (attrkey, _derivexsdtype(attrvalue))
					elif attrkey[0] == '_':
						pass
					else:
						if isListOrTuple(attrvalue) and hasattr(__object, "__xmlseqnamesdict__") and attrkey in __object.__xmlseqnamesdict__.keys():  # It's a list, but we want each item in the list to have a special element name
							subkey = key = __object.__xmlseqnamesdict__[attrkey]
						text += createxsd(attrvalue, key = attrkey, subkey = subkey, nestLevel = nestLevel + 2)
				text += INDENT_CHARS * (nestLevel + 1) + "</xs:complexType>\n"
				text += indent + "</xs:element>\n"

	if nestLevel == 0:
		text += "</xs:schema>\n"

	return text

def createxsd(object, key = '', subkey = '', nestLevel = 0):
	if nestLevel == 0:
		text = """<?xml version="1.0" encoding="ISO-8859-1" ?>\n<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">\n"""
	else:
		text = ''
	indent = INDENT_CHARS * nestLevel

	if not object:
		return ''

	if not key:
		if hasattr(object, "__xmlname__"):
			key = object.__xmlname__
		else:
			key = str(object.__class__).lower()

	text += indent + "<xs:element name=\"%s\"" % key
	if isSimpleType(object):
		text += " type=\"%s\"/>\n" % _derivexsdtype(object)
	else:
		text += '>\n'
		if isListOrTuple(object) and len(object) > 0:  # List
			text += INDENT_CHARS * (nestLevel + 1) + "<xs:complexType>\n"
			text += INDENT_CHARS * (nestLevel + 2) + "<xs:sequence>\n"
			if subkey:
				keytouse = subkey
			else:
				keytouse = key
			text += createxsd(object[0], key = keytouse, nestLevel = nestLevel + 3)
			text += INDENT_CHARS * (nestLevel + 2) + "</xs:sequence>\n"
			text += INDENT_CHARS * (nestLevel + 1) + "</xs:complexType>\n"
			text += indent + "</xs:element>\n"
		elif isinstance(object, type({})):  # Dict
			pass  # TODO
		else:  # It's an object, nest into it
			text += INDENT_CHARS * (nestLevel + 1) + "<xs:complexType"
			if hasattr(object, "__xmlcomplextypename__"):
				text += " name=\"%s\"" % object.__xmlcomplextypename__
			text += '>\n'
			for (attrkey, attrvalue) in object.__dict__.items():
				origattrkey = attrkey
				attrkey = attrkey.lower()
				if hasattr(object, "__xmlmappingdict__") and object.__xmlmappingdict__.has_key(origattrkey):
					attrkey = object.__xmlmappingdict__[origattrkey]

				if hasattr(object, "__xmlexclude__") and origattrkey in object.__xmlexclude__:
					pass
				elif hasattr(object, "__xmlattr__") and origattrkey in object.__xmlattr__:
					text += INDENT_CHARS * (nestLevel + 2) + "<xs:attribute name=\"%s\" type=\"%s\"/>\n" % (attrkey, _derivexsdtype(attrvalue))
				elif attrkey[0] == '_':
					pass
				else:
					if isListOrTuple(attrvalue) and hasattr(object, "__xmlseqnamesdict__") and attrkey in object.__xmlseqnamesdict__.keys():  # It's a list, but we want each item in the list to have a special element name
						subkey = key = object.__xmlseqnamesdict__[attrkey]
					text += createxsd(attrvalue, key = attrkey, subkey = subkey, nestLevel = nestLevel + 2)
			text += INDENT_CHARS * (nestLevel + 1) + "</xs:complexType>\n"
			text += indent + "</xs:element>\n"

	if nestLevel == 0:
		text += "</xs:schema>\n"

	return text

def _derivexsdtype(value):
	if isinstance(value, type('')) or isinstance(value, type(u'')):
		return "string"
	elif isinstance(value, type(123)):
		return "integer"
	elif isinstance(value, type(123.1)):
		return "float"
	elif isinstance(value, datetime.date):
		return "date"
	elif isinstance(value, datetime.datetime):
		return "dateTime"
	elif isinstance(value, datetime.time):
		return "time"
	return "complexType"

def _castxsdvalue(value, xsdType):
	try:
		if xsdType == "integer":
			return int(value)
		elif xsdType == "float":
			return float(value)
		elif xsdType == "date":
			return datetime.datetime(int(value[:4]), int(value[5:7]), int(value[8:10]))
		elif xsdType == "dateTime":
			if len(value) == 19:  # No microseconds
				return datetime.time(int(value[:4]), int(value[5:7]), int(value[8:10]), int(value[11:13]), int(value[14:16]), int(value[17:19]))
			elif len(value) == 26:  # With microseconds
				return datetime.time(int(value[:4]), int(value[5:7]), int(value[8:10]), int(value[11:13]), int(value[14:16]), int(value[17:19]), int(value[20:27]))
		elif xsdType == "time":
			if len(value) == 8:  # No microseconds
				return datetime.time(int(value[:2]), int(value[3:5]), int(value[6:8]))
			elif len(value) == 15:  # With microseconds
				return datetime.time(int(value[:2]), int(value[3:5]), int(value[6:8]), int(value[9:15]))
		else:
			return value
	except: # Just use the string value if there is some type of parsing error
		return value

