# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, sre, os, urllib, xmlrpclib, time, glob
import Conf

import i18n
_ = i18n.gettext()

def is_valid_command (command) :
	if command.count(os.sep) > 0 :
		if not os.path.exists(command) :
			return False
		else :
			return True

	else :
		_i, _o, _e = os.popen3("/usr/bin/which %s" % command)

		if len(_o.readlines()) < 1 :
			return False
		else :
			return True

def escape_path (path) :
	return path.replace( \
		" ", "\\ ").replace( \
		"#", "\\#").replace( \
		"%", "\\%").replace( \
		"(", "\\(").replace( \
		")", "\\)")

import sre
def get_protocol_from_uri (uri) :
	__a = uri.split("://")

	if len(__a) < 2 : return (None, uri)
	return (__a[0], "".join(__a[1:]), )

def get_filename_from_uri (uri) :
	"""
	get the unix file path from uri
	"""

	__a = uri.split(":")
	if len(__a) < 2 : return uri

	slash = sre.compile("^//")
	__b = urllib.url2pathname(slash.sub("", __a[1])).strip()

	return __b

def parse_command_line_argument (command) :
	parsed_data = { \
		"environment" : tuple(),
		"command" : str(),
		"argument" : tuple(),
	}

	__a = command.split()

	if len(__a) < 2 :
		parsed_data["command"] = command.strip()
		return parsed_data

	q = sre.compile("[A-Z][A-Z_][A-Z_]*=.*") # env

	__env = list()
	__index_env = list()
	for i in range(len(__a)) :
		if q.findall(__a[i]) :
			__env.append(__a[i].strip())
			__index_env.append(i)
		else :
			break

	parsed_data["environment"] = tuple(__env)

	if len(__index_env) > 0 :
		__index = __index_env[-1]
	else :
		__index = -1

	try :
		__command = __a[__index + 1]
	except IndexError :
		return parsed_data

	parsed_data["command"] = __command

	try :
		__argv = __a[__index + 2:]
	except IndexError :
		return parsed_data

	parsed_data["argument"] = tuple(__argv)

	return parsed_data

def normalize_path (path) :
	return path.replace("~", os.getenv("HOME"))

def get_labelname (label_name, size=10) :
	if label_name.count(os.sep) < 1 and size is not None:
		try :
			return label_name[-size]
		except IndexError :
			return label_name
	else :
		__filename = os.path.basename(label_name)
		__pathname = os.path.dirname(label_name)

		if len(__filename) >= size or size is None :
			return __filename
		else :
			__a = __pathname[-(size - len(__filename)):]

			return os.path.join(__a, __filename)

import gtk
def color_to_str (gdkColor) :
	if not isinstance(gdkColor, gtk.gdk.Color) :
		raise ValueError, \
			_("gdkColor must be the instance of gtk.gdk.Color.")

	color = "#%2X%2X%2X" % ( \
		gdkColor.red / 256, \
		gdkColor.green / 256, \
		gdkColor.blue / 256, \
	)

	return color.replace(" ", "0")

import gtk
def str_to_color (color_spec, default="white") :
	try :
		return gtk.gdk.color_parse(color_spec)
	except ValueError :
		return gtk.gdk.color_parse(default)
	except TypeError :
		return None

def is_valid_pid (pid) :
	if sys.platform == "linux2" :
		__directory_pid = "/proc/%d" % pid
		if not os.path.isdir(__directory_pid) :
			return False
		else :
			return True
	else :
		return True

import gtk
def create_icon (icon_file_path, w=30, h=30) :
	"""
	@return pixbuf image.
	"""
	if not os.path.isfile(icon_file_path) :
		return None

	__i = gtk.Image()
	__i.set_from_file(icon_file_path)
	__i.set_property("icon-size", 0)

	__image = __i.get_pixbuf().scale_simple(w, h, 2)

	return __image

def parse_geometry (geom) :
	if geom.count("x") == 0 :
		w, h = None, None
	else :
		__wh = sre.compile("[1-9][0-9]*x[1-9][0-9]*").findall(geom)

		try :
			w, h = __wh[0].split("x")
		except IndexError :
			return None
		else :
			w, h = (int(w), int(h),)

	if geom.count("+") == 0 :
		x, y = None, None
	else :
		__xy = sre.compile("\+[1-9][0-9]*\+[1-9][0-9]*").findall(geom)

		try :
			x, y = __xy[0].replace("+", " ").split()
		except IndexError :
			return None
		else :
			x, y = (int(x), int(y),)

	return (w, h, x, y)

def remember_geometry (window, gconf_client, is_fullscreened=False) :
	if is_fullscreened : window.unfullscreen()

	(w, h) = window.get_data("window-size")
	(x, y) = window.get_position()

	gconf_client.set_value_by_conf( \
		"latest-window-geometry", \
		"%d,%d,%d,%d" % (w, h, x, y) \
	)

	# save the state of fullscreened or not.
	gconf_client.set_value_by_conf( \
		"is-fullscreen", is_fullscreened)

def get_parent_directory (path) :
	if not os.path.isdir(path) : return None

	return os.path.abspath(os.path.join(path, os.pardir))

def get_parent_directories (path) :
	__path = get_parent_directory(path)
	return get_subdirectories(__path)

def has_parent_directory (path) :
	if not os.path.isdir(path) : return False

	if len(get_parent_directories(path)) < 1 : return False
	else : return True

def get_subdirectories (path) :
	try :
		return filter(lambda i : os.path.isdir(i), \
			[ os.path.join(path, i)  for i in os.listdir(path) ])
	except OSError :
		return list()

def has_subdirectory (path) :
	if not os.path.isdir(path) : return False

	if len(get_subdirectories(path)) < 1 : return False
	else : return True

def get_pid_by_process_name (name) :
	cmd = "COLUMNS=10000 ps aux | grep \"%s\" | grep -v grep | awk '{print $2}'" % name

	i, o, e = os.popen3(cmd)
	__a = o.readlines()

	if len(__a) != 1 : return None

	try :
		__pid = int(__a[0].strip())
	except ValueError :
		return None

	return __pid

def casting (type, value) :

	if type == "str" :
		return str("%s" % value)
	elif type == "int" :
		if not value :
			return int()
		else :
			value = str(value)
			return int(value.split(".")[:1][0])
	elif type == "float" :
		if not value :
			return float()
		else :
			return float("%s" % value)
	elif type == "bool" :
		return bool(value)
	else :
		return value

def get_formatted_time (timestamp, format="") :

	"""
	time formatting
	-----------------
	 %c							: Sun Mar 20 01:51 2005
	 "%Y-%m-%d %H:%M	: 2005-03-20 01:51
	 Today %p %l:%M	: Today AM 1:52
	"""

	try :
		timestamp = int(timestamp)
	except ValueError :
		return ""
	except TypeError :
		return ""

	if not format.strip() : format = "Today %p %l:%M"

	if format == "Today %p %l:%M" :
		return get_formatted_time_friendly(timestamp)
	elif format == "%c" :
		return get_formatted_time_fully(timestamp)
 	elif format == "%Y-%m-%d %H:%M" :
		return get_formatted_time_digit(timestamp)
	else : # default is "Today %p %l:%M"
		return get_formatted_time_friendly(timestamp)

def get_formatted_time_friendly (timestamp) :

	format = "Today %p %l:%M"

	__current_time_tuple = int(time.time())

	elapsed = __current_time_tuple - timestamp

	if elapsed < 60*60*24 : # today.
		format = sre.compile("today",sre.IGNORECASE).sub(_("Today"), format)
	elif elapsed < 60*60*24*2 : # yesterday.
		format = sre.compile("today",sre.IGNORECASE).sub(_("Yesterday"), format)
	else :
		__d =  _("day")
		if __d != "day" : __d
		else : __d = ""

		format = "%%B %%e%s %%p %%l:%%M %%Y" % __d

	return time.strftime(format, time.localtime(timestamp))

def get_formatted_time_fully (timestamp) :
	return time.strftime("%Y-%m-%d %H:%M", time.localtime(timestamp))

def get_formatted_time_digit (timestamp) :
	return time.strftime("%c", time.localtime(timestamp))

def get_elapsed_time_old (timestamp) :
	"""
	Calculate the elapsed time.
	@return : ()
	"""

	try :
		timestamp = int(timestamp)
	except ValueError :
		return ""
	except TypeError :
		return ""

	a = time.localtime((time.time() - timestamp) + \
		time.mktime((1970,1,1,1,1,1,1,1,0)))

	__l = list()
	[ __l.append(i - 1) for i in a[1:] ]
	__a = ([(a[0] - 1970), ]  + __l)[:-3]

	__time = list()
	if __a[4] <= 0 and __a[3] <= 0 and __a[2] <= 0 and \
			__a[1] <= 0 and __a[0] <= 0 :
		__time.append(_("%d seconds") % __a[5])

	if __a[4] > 0 : __time.append(_("%d minutes") % __a[4])
	if __a[3] > 0 : __time.append(_("%d hours") % __a[3])
	if __a[2] > 0 : __time.append(_("%d days") % __a[2])
	if __a[1] > 0 : __time.append(_("%d months") % __a[1])
	if __a[0] > 0 : __time.append(_("%d years") % __a[0])

	__time.reverse()
	return _("%s ago") % (" ".join(__time))

def get_elapsed_time (timestamp) :
	"""
	Calculate the elapsed time.
	@return : ()
	"""

	try :
		timestamp = int(timestamp)
	except ValueError :
		return ""
	except TypeError :
		return ""

	(__d, __h, __m, __s) = timediff(timestamp)

	__Y = 0
	__M = 0
	if __d > 365 :
		__Y = int(__d / 365)
		__d = __d % 365

	if __d > 30 : # or 31?
		__M = int(__d / 30)
		__d = int(__d % 30)

	__time = list()
	if __d < 1 and __h < 1 and __m < 1 :
		__time.append(_("%d seconds") % __s)

	if __m > 0 :
		__time.append(_("%d minutes") % __m)
	if __h > 0 : __time.append(_("%d hours") % __h)
	if __d > 0 : __time.append(_("%d days") % __d)
	if __M > 0 : __time.append(_("%d months") % __M)
	if __Y > 0 : __time.append(_("%d years") % __Y)

	__time.reverse()
	return _("%s ago") % (" ".join(__time))

import time, datetime

def timediff (timestamp) :
	"""
	Get the elapsed time by seconds.
	"""

	__a = datetime.timedelta(seconds=int(time.time()) - timestamp)

	h = 0; m = 0; s = 0

	__hours = __a.seconds / (60 * 60)
	if __hours < 1 :
		__min = __a.seconds
	else :
		h = __hours
		__min = __a.seconds % (60 * 60)

	__min = __min / 60
	if __min < 1 :
		s = __a.seconds
	else :
		m = __min
		s = __a.seconds % 60

	return (__a.days, h, m, s, )

def cleanup_temporary_directory (tmp_directory) :
	for i in glob.glob("%s/*" % tmp_directory) :
		try :
			os.remove(i)
		except OSError :
			continue

import urllib

def quote_filename (title) :
	return urllib.pathname2url(str(title)).replace("%", "_")

def unquote_filename (title) :
	return urllib.url2pathname(str(title).replace("_", "%"))

import random
def get_uid_old () :
	return random.random().__str__()

import time, random, socket, md5
def get_uid (args=str()) :
	"""
	Generates a universally unique ID.
	Any arguments only create more randomness.

	----------------------------------------------------------------------
	Written by Carl Free Jr.
	url : http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/213761
	----------------------------------------------------------------------
	"""

	t = long( time.time() * 1000 )
	r = long( random.random()*100000000000000000L )
	try:
		a = socket.gethostbyname( socket.gethostname() )
	except:
		# if we can't get a network address, just imagine one
		a = random.random() * 100000000000000000L
	data = str(t) + ' ' + str(r) + ' ' + str(a) + ' ' + args
	data = md5.md5(data).hexdigest()
	return data

import gtk
def get_pixbuf_filled_with_color (spec, (width, height,)) :
	"""
	Create a gtk.Image object, which was filled with one color.
	If spec is None, fully transparent pixbuf will be returned.

	@args :
		str spec : a string containing a color specification
		int width : image width
		int height : image height

	@return : gtk.gdk.Pixbuf
	"""

	__is_alpha = False

	colormap = gtk.gdk.colormap_get_system()
	try :
		__color = colormap.alloc_color(gtk.gdk.color_parse(spec))
	except ValueError :
		__is_alpha = True
	except TypeError :
		__is_alpha = True

	if __is_alpha :
		__color = colormap.alloc_color(gtk.gdk.color_parse("#FFFFFF"))

	__gdk_image = gtk.gdk.Image( \
		gtk.gdk.IMAGE_FASTEST, gtk.gdk.visual_get_best(), width, height)
	for i in range(width) :
		for j in range(height) :
			__gdk_image.put_pixel(i, j, long(__color.pixel))

	__pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, width, height)
	__pixbuf.get_from_image(__gdk_image, colormap, 0, 0, 0, 0, width, height)

	if __is_alpha :
		__pixbuf = __pixbuf.add_alpha(True, chr(0xFF),chr(0xFF),chr(0xFF))

	return __pixbuf

# GTK MODIFIER MASK list
NULL_MODIFIER_MASK = 0

AVAILABLE_MODIFIER_MASK = \
	gtk.gdk.SHIFT_MASK | \
	gtk.gdk.LOCK_MASK | \
	gtk.gdk.CONTROL_MASK | \
	gtk.gdk.MOD1_MASK | \
	gtk.gdk.MOD2_MASK | \
	gtk.gdk.MOD3_MASK | \
	gtk.gdk.MOD4_MASK | \
	gtk.gdk.MOD5_MASK
	#gtk.gdk.BUTTON1_MASK | \
	#gtk.gdk.BUTTON2_MASK | \
	#gtk.gdk.BUTTON3_MASK | \
	#gtk.gdk.BUTTON4_MASK | \
	#gtk.gdk.BUTTON5_MASK | \
	#gtk.gdk.RELEASE_MASK | \
	#gtk.gdk.MODIFIER_MASK

UPPER_SYMBOLS = { \
	"exclam" : "1",                 	"braceleft" : "7", \
	"numbersign" : "3",             	"Control_underscore" : "7", \
	"percent" : "5",                	"asterisk" : "8", \
	"asciicircum" : "6",            	"bracketleft" : "8", \
	"plus" : "equal",               	"Delete" : "8", \
	"braceleft" : "bracketleft",    	"parenleft" : "9", \
	"colon" : "semicolon",          	"bracketright" : "9", \
	"quotedbl" : "apostrophe",      	"parenright" : "0", \
	"asciitilde" : "grave",         	"braceright" : "0", \
	"bar" : "backslash",            	"underscore" : "minus", \
	"less" : "comma",               	"backslash" : "minus", \
	"greater" : "period",           	"Control_underscore" : "minus", \
	"question" : "slash",           	"Control_underscore" : "minus", \
	"dollar" : "4",                 	"braceright" : "bracketright", \
	"Control_backslash" : "4",      	"asciitilde" : "bracketright", \
	"at" : "2",                     	"Control_bracketright" : "bracketright", \
	"nul" : "2",                    	"greater" : "less", \
	"ampersand" : "7",              	"bar" : "less", \
}

import sre
def get_keyname (event, **kwargs) :
	"""
	Keyword arguments,
		ignore_shift
			: when typing a upper letter, ignore <Shift>.
	"""

	__IS_IGNORE_SHIFT = False
	if kwargs.has_key("ignore_shift") and kwargs["ignore_shift"] :
		__IS_IGNORE_SHIFT = True

	__keyval_name = gtk.gdk.keyval_name(event.keyval)

	#__mod = event.state & gtk.gdk.MODIFIER_MASK
	__mod = event.state & AVAILABLE_MODIFIER_MASK
	if __mod == NULL_MODIFIER_MASK :
		return __keyval_name

	if not __keyval_name : return ""
	if not __keyval_name.strip() : return ""

	if not gtk.accelerator_valid(event.keyval, event.state) :
		return ""

	# check the symbols caused by SHIFT key.
	__key_string = gtk.gdk.keyval_name(event.keyval)
	if UPPER_SYMBOLS.has_key(__key_string) :
		mods = event.state & ~ gtk.gdk.SHIFT_MASK
	else :
		if __IS_IGNORE_SHIFT and \
				sre.compile("[a-zA-Z]").findall(__key_string.lower()) :
			mods = event.state
			if mods == gtk.gdk.SHIFT_MASK :
				return __key_string.lower()
		else :
			mods = event.state

	# store key event.
	if len(__key_string) == 1 : __key_string = __key_string.upper()

	__key_name = gtk.accelerator_name(event.keyval, mods)

	# filtering the close this window.
	if not mods :
		__key_value = __key_string
	else :
		__key_value = ">".join(__key_name.split(">")[:-1]) + ">" + \
			__key_string

	return __key_value

import os
def get_file_extention (filename) :
	"""
	Get the file type simply by extention string.

	if ext is,
		(png|tiff|gif|jpg|jpeg) : image
		(py|c|txt|java|php|conf) and etc. : text file
		(tar|gz|[zZ]zip|rar) : it's compressed
		...
	"""
	__f = os.path.basename(filename)

	# get extention.
	__ext = __f.split(".")[-1]

	if __ext in ["png", "tiff", "gif", "jpg", "jpeg", ] :
		return "image"
	elif __ext in ["py", "c", "txt", "java", "php", "conf", ] :
		return "plain"
	elif __ext in ["tar", "gz", "z", "Z", "zip", "rar", ] :
		return "compress"
	else :
		return str()

import string
def escape_html (buffer):
	return string.replace(
	    string.replace( \
	    string.replace( \
	    string.replace( \
		 	buffer,
				'&', '&amp;'), \
				'<', '&lt;'), \
				'>', '&gt;'), \
				'"', '&quot;' \
		)

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "func.py"


